"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.InternalProvider = void 0;
var Errors_1 = require("./Errors");
var InternalProvider = /** @class */ (function () {
    function InternalProvider(container, create, validation, context) {
        this._ = {
            context: context,
            validation: validation,
            container: container,
            create: create,
        };
    }
    InternalProvider.prototype.get = function (nameSelector) {
        var _a;
        var _b = this._, container = _b.container, context = _b.context, create = _b.create, validation = _b.validation;
        var name = container.resolveProperty(nameSelector);
        try {
            var actual = context ? this : create(validation, {});
            return (_a = container.get(name)) === null || _a === void 0 ? void 0 : _a.provide(actual);
        }
        catch (e) {
            var error = e;
            switch (error.type) {
                case Errors_1.DependencyErrorType.Circular:
                    // When error is detected we can only say lifetime === cause
                    // Letting it throw 1 dependency up we can find the dependencies causing the issue
                    if (error.lifetime !== error.cause)
                        throw e;
                    throw new Errors_1.CircularDependencyError(error.lifetime, name);
                default:
                    throw e;
            }
        }
    };
    InternalProvider.prototype.validate = function () {
        var template = this._.container.template;
        var keys = Object.keys(template);
        var unresolved = [];
        for (var _i = 0, keys_1 = keys; _i < keys_1.length; _i++) {
            var key = keys_1[_i];
            try {
                this.get(key);
            }
            catch (e) {
                unresolved.push(e);
            }
        }
        if (unresolved.length === 0)
            return;
        throw new Errors_1.MultiDependencyError(unresolved);
    };
    return InternalProvider;
}());
exports.InternalProvider = InternalProvider;
